package FigCommon::SwitchRelease;
#
# switch_to_release <release_number>
#
# Change the currently-running release to <release_number>
#
# Check to be sure that $fig_disk/dist/releases/<release_number> exists
#
# Update $fig_disk/CURRENT_RELEASE with the new release number.
#
# Swing the symlinks $fig_disk/FIG/bin and $fig_disk/FIG/CGI to the right place
#
# Create the symlink $fig_disk/disk/releases/current to point to <release_number>
#
# Expand the environment-specific tool headers from $fig_disk/config/base_tool_hdr
# to include the release-specific directories. Place the output in
# dist/releases/<release_number>/<arch>.
#

sub switch_to_release
{
    my($fig_disk, $arch, $release_number) = @_;

    my $release_dir = "$fig_disk/dist/releases/$release_number";


    if (! -d $release_dir)
    {
	die "Release directory $release_dir does not exist"
	}

    #
    # Okay, go for it.
    #
    
    if (!open(FH, ">$fig_disk/CURRENT_RELEASE"))
    {
	die "Cannot write to $fig_disk/CURRENT_RELEASE";
    }
    print FH "$release_number\n";
    close(FH);
    
    #
    # Set up the arch-specific directory in the release directory
    # if it's not already there.
    #

    if (! -d "$release_dir/$arch")
    {
	print "Creating $release_dir/$arch\n";
	mkdir("$release_dir/$arch");
    }

    #
    # Symlinks.
    #

    my $bindir = "$fig_disk/FIG/bin";
    my $cgidir = "$fig_disk/FIG/CGI";

    #
    # See if bin and/or CGI are directories; they are if this is an old-style
    # FIG setup. Rename them and do the symlink.
    #

    if (-d $bindir && ! -l $bindir)
    {
	my $bak = "$bindir." . time();
	warn "Renaming $bindir to $bak\n";
	rename($bindir, $bak);
    }
    if (-d $cgidir && ! -l $cgidir)
    {
	my $bak = "$cgidir." . time();
	warn "Renaming $cgidir to $bak\n";
	rename($cgidir, $bak);
    }
    
    unlink("$fig_disk/FIG/bin");
    symlink("../dist/releases/$release_number/$arch/bin", "$fig_disk/FIG/bin") ;
    unlink("$fig_disk/FIG/CGI");
    symlink("../dist/releases/$release_number/$arch/CGI", "$fig_disk/FIG/CGI");
    unlink("$fig_disk/FIG/Tutorial");
    symlink("../dist/releases/$release_number/$arch/Tutorial", "$fig_disk/FIG/Tutorial");

    unlink("$fig_disk/dist/releases/current");
    symlink($release_number, "$fig_disk/dist/releases/current");

    #
    # Expand the tool headers.
    #
    
    expand_header("$fig_disk/config/base_tool_hdr",
		  "$release_dir/$arch/tool_hdr",
		  <<END);
use lib '$release_dir/$arch/lib';
use lib '$release_dir/$arch/lib/FigKernelPackages';
END
    
    expand_header("$fig_disk/config/base_tool_hdr_py",
		  "$release_dir/$arch/tool_hdr_py",
		  <<END);
import os
os.environ["PERL5LIB"] = ":".join([os.getenv("PERL5LIB", ""), '$release_dir/$arch/lib', '$release_dir/$arch/lib/FigKernelPackages'])
sys.path.append('$release_dir/$arch/lib')
END
    

    &run_make($fig_disk, $arch, $release_number, $release_dir);

    #
    # Fix up permissions so we can do things as a different user later.
    #

    chmod(0777, $fig_disk);
    chmod(0777, "$fig_disk/FIG");
    chmod(0777, "$fig_disk/FIG/bin");
    chmod(0777, "$fig_disk/FIG/CGI");

    system("chmod ugo+w $fig_disk");
    system("chmod ugo+w $fig_disk/CURRENT_RELEASE");
    system("chmod ugo+w $fig_disk/FIG");
    system("chmod ugo+w $fig_disk/FIG/bin");
    system("chmod ugo+w $fig_disk/FIG/CGI");
    system("chmod ugo+w $fig_disk/dist/releases");
    system("chmod  -R ugo+w $release_dir");
}

#
#
# Run a make in the release.
#
# This sets up the local environment to properly do the make.
#

sub run_make
{
    my($fig_disk, $arch, $release_number, $release_dir) = @_;

    my %save_env = %ENV;

    $ENV{RTCURRENT} = $release_number;
    $ENV{RTROOT} = $fig_disk;
    $ENV{FIG_HOME} = $fig_disk;
    $ENV{RTDIST} = "$fig_disk/dist";
    $ENV{RTARCH} = $arch;

    system("cd $release_dir; make clean; make");

    %ENV = %save_env;
}

sub expand_header
{
    my($input, $output, $text) = @_;

    my($fh_in, $fh_out);

    open($fh_in, "<$input") or
	die "expand_header: cannot open $input: $!";
    open($fh_out, ">$output") or
	die "expand_header: cannot open $output: $!";
    for (<$fh_in>)
    {
	if (/^#BEGIN switch_to_release generated code/)
	{
	    print $fh_out $_;
	    while (<$fh_in>)
	    {
		if (/^#END switch_to_release generated code/)
		{
		    print $fh_out $_;
		    last;
		}
	    }
	    print $fh_out $text;
	}
	else
	{
	    print $fh_out $_;
	}
    }
    close($fh_in);
    close($fh_out);
}




1;
	
