#
# Postgres configuration.
#
# Determine the identifier for this seed install.
# For unix-like systems, it's the inode of the directory holding
# the FIGdisk.
#

print "In envsub postgres\n";

#
# Determine figdisk identifier.
#

my $fig_ident;

my @s = stat($fig_disk);
$fig_ident = $s[1];
if ($fig_ident == 0)
{
    #
    # Stat didn't work. Identify as the pathname.
    #

    $fig_ident = $fig_disk;
    $fig_ident =~ s,\W,,g;
}
print "Have fig_ident=$fig_ident\n";

#
# Create the db directory if necessary
#

my $dbdir = "$ENV{HOME}/FIGdb";
if (! -d $dbdir)
{
    mkdir($dbdir) or die "Could not create database directory $dbdir";
}

#
# Look for initdb in the environment dir, or in the path  if it's not there.
#

my $initdb = "$env_bin/initdb";
if (! -x $initdb)
{
    $initdb = find_in_path("initdb");
}

if (! -x $initdb)
{
    die "Could not find Postgres initialization program 'initdb'.";
}

my $postmaster = "$env_bin/postmaster";
if (! -x $postmaster)
{
    $postmaster = find_in_path("postmaster");
}

if (! -x $postmaster)
{
    die "Could not find Postgres postmaster.";
}

my $pg_ctl = "$env_bin/pg_ctl";
if (! -x $pg_ctl)
{
    $pg_ctl = find_in_path("pg_ctl");
}

if (! -x $pg_ctl)
{
    die "Could not find Postgres pg_ctl.";
}

#
# Configure pgdata directory
#

my $pgdata = "$dbdir/$fig_ident";
    
if (-d $pgdata)
{
    #
    # pg database directory already there. Look to see if it appears
    # to be the right one for this version of postgres.
    #

    my $version;
    
    open(FH, "$initdb --version|") or die "Cannot run $initdb: $!\n";
    $_ = <FH>;
    close(FH);
    chop;
    if (/initdb.* (\d\.\d)/)
    {
	$version = $1;
    }
    else
    {
	warn "Cannot determine postgres version from output: $_\n";
    }
	
    open(FH, "<$pgdata/PG_VERSION");
    my $dbver = <FH>;
    chop($dbver);
    close(FH);

    if ($dbver != $version)
    {
	warn "Existing pgdata directory $pgdata is for the wrong version; moving out of the way.";

	rename($pgdata, "$pgdata.bak.$$");
    }
}

if (! -d $pgdata)
{
    mkdir($pgdata) or die "Could not create database directory $pgdata";

    #
    # Initialize the db directory.
    #
    
    #
    # Find the share directory where postgres.bki lives.
    #

    my @possible_dirs = ("$env_dir/share/postgresql", "$env_dir/share");

    my $share;
    for my $d (@possible_dirs)
    {
	if (-f "$d/postgres.bki")
	{
	    $share = $d;
	    last;
	}
    }

    if (!$share)
    {
	die "Couldn't find postgres shared data directory\n";
    }

    if (!open(IN, "$initdb --locale=C -D $pgdata -L $share |"))
    {
	die "Execution of $initdb failed: $!\n";
    }

    while (<IN>)
    {
	chomp;
	print "Got $_\n";
    }

    if (!close(IN))
    {
	if ($! == 0)
	{
	    #
	    # Nonzero exit code
	    #
	    
	    my $rc = $? >> 8;
	    die "$initdb exited with return code $rc";
	}
	else
	{
	    die "$initdb failed: $!\n";
	}
    }
}

#
# Okay, database initialized.
#
# Update the index file in the figdb dir so people know
# what's going on.
#

if (! -f "$dbdir/README")
{
    open(F, ">$dbdir/README");
    print F <<END;
This is the FIG Postgres data directory. Subdirectories contain
Postgres database clusters for SEED installations.

Cluster directory                SEED directory
END
}
else
{
    open(F, ">>$dbdir/README");
}

my $mywd = `/bin/pwd`;
chop($mywd);
printf F "%-32s %s %s\n", $pgdata, $ENV{FIG_HOME}, $mywd;

close(F);

#
# We should be ready to go. Put the config file directives in.
#

#
# We use the current account as the db user.
#

my $dbuser = (getpwuid($>))[0];
my $dbport = 10000;
my $logfile = "$dbdir/postgres.log";
my $dbname = "fig";
my $dbbufs = 100;

#
# Compute postgres shared buffer sizes. 
#
# We'd like to have at least 4700 buffers (that seems enough
# with current (jan 2004) data sizes to make the postgres
# backend run in RAM and not page.
#

#
# These routines come from formulas published in the Postgres
# documentation, section 16.5.1:
#
#	http://www.postgresql.org/docs/current/static/kernel-resources.html
# 

sub req_shm
{
    my($bufs, $max_conn) = @_;

    my $shm = 250 *1024 + 8.2 * 1024 * $bufs + 14.2 *1024 * $max_conn;
    return int($shm);
}

sub bufs_from_shm
{
    my($shm, $max_conn) = @_;

    my $bufs = ($shm - 250 * 1024 - 14.2 * 1024 * $max_conn) / (8.2 * 1024);

    return int($bufs);
}

my $physmem = $OtherConfig->{memory};
my $shmmax = $OtherConfig->{shmmax};

if ($physmem != -1 && $shmmax != -1)
{
    my $max_conn = 100; 	# Postgres default.
    my $mem_wanted;

    #
    # Cap shared memory at 25% of physical memory.
    #
    if ($shmmax > (0.25 * $physmem))
    {
	$mem_wanted = int(0.25 * $physmem);
    }
    else
    {
	$mem_wanted = $shmmax;
    }
    my $max_bufs = &bufs_from_shm($mem_wanted, $max_conn);

    #
    # Back it off a bit to make sure it'll start.
    #
    $max_bufs = int(0.95 * $max_bufs);

    print "Computed max_bufs=$max_bufs from physmem=$physmem and shmmax=$shmmax\n";
    $dbbufs = $max_bufs;
}

#
# This is pretty ugly, but necessary sometimes.
#
# Override defaults with FIGCONFIG_XX
#

if ($ENV{FIGCONFIG_DBNAME} ne "")
{
    $dbname = $ENV{FIGCONFIG_DBNAME};
}

if ($ENV{FIGCONFIG_DBUSER} ne "")
{
    $dbuser = $ENV{FIGCONFIG_DBUSER};
}

if ($ENV{FIGCONFIG_DBPORT} ne "")
{
    $dbport = $ENV{FIGCONFIG_DBPORT};
}

if ($ENV{FIGCONFIG_DBBUFS} ne "")
{
    $dbbufs = $ENV{FIGCONFIG_DBBUFS};
}

push(@$Config, ["dbms", "Pg", 1]);
push(@$Config, ["db", $dbname, 1]);
push(@$Config, ["dbuser", $dbuser, 1]);
push(@$Config, ["dbpass", "", 1]);
push(@$Config, ["dbport", $dbport, 0]);

push(@$Env, ["PGDATA", $pgdata]);
push(@$Env, ["PGPORT", $dbport]);

#
# Create the start/stop servers script
#

open(FH, ">>$start_servers");
print FH "$postmaster -B $dbbufs -p $dbport -D $pgdata >$logfile 2>&1 </dev/null &\n";
close(FH);

open(FH, ">>$stop_servers");
print FH "$pg_ctl -D $pgdata -w stop\n";
close(FH);


1;
